package com.icontrols.kakao.socket.wallpad;

import java.net.URI;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Base64;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.concurrent.CompletableFuture;

import javax.annotation.Resource;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.apache.http.conn.ssl.NoopHostnameVerifier;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.http.client.HttpComponentsClientHttpRequestFactory;
import org.springframework.stereotype.Service;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestTemplate;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.icontrols.kakao.common.ERROR_INFO;
import com.icontrols.kakao.common.KAKAO_RESULT;
import com.icontrols.kakao.common.KEY_INFO;
import com.icontrols.kakao.common.SERVERINFO;
import com.icontrols.kakao.common.THINGS_INFO;
import com.icontrols.kakao.common.TYPE;
import com.icontrols.kakao.common.TYPE.XML_LIGHT;
import com.icontrols.kakao.common.utils.BeanUtils;
import com.icontrols.kakao.common.utils.CommonUtil;
import com.icontrols.kakao.config.AsyncConfig;
import com.icontrols.kakao.entity.DonghoInfo;
import com.icontrols.kakao.repo.DonghoInfoRepository;
import com.icontrols.kakao.socket.utils.IPGenerator;
import com.icontrols.kakao.socket.utils.SSLClient;
import com.icontrols.kakao.socket.utils.XMLConstants.REMOTE_KIND;
import com.icontrols.kakao.socket.utils.XMLGenerator;

@Service("kakaoMsgHandler")
public class KakaoMsgHandler {
	private final static Logger logger = LoggerFactory.getLogger(KakaoMsgHandler.class);
	
	@Resource(name="asyncConfig")
	private AsyncConfig asyncConfig;
	
	@Resource(name="wallpadMsgSender")
	WallpadMsgSender wallpadMsgSender;
	
	/**
	 * 카카오 전용 인증 프로세스
	 * 월패드에 인증번호를 노출시키는 메시지를 전송한다.
	 * @param param
	 * @return
	 */
	public Map<String, Object> doAuthProcess(Map<String, Object> param){
		Map<String, Object> retMap = new HashMap<String, Object>();
		
		try {
			int dong = Integer.parseInt((String)param.get("dong"));
			int ho = Integer.parseInt((String)param.get("ho"));
			String code = (String)param.get("code");
			
			String targetIP = IPGenerator.getIPFromDongHo(dong, ho);
			String twinid = UUID.randomUUID().toString();
			String alias = "Kakao";
			StringBuffer sb = new StringBuffer();
			
			sb.append(XMLGenerator.MakeXMLHeaderForNotice(REMOTE_KIND.remote_register_new_device_auth_number.getContents()))
			.append("      <home_info dong=\"").append(dong).append("\" ho=\"").append(ho).append("\" ip=\"").append(targetIP).append("\" type=\"main\"/>\r\n")
			.append("      <device_info alias=\"").append(alias).append("\" twinid=\"").append(twinid).append("\" auth_number=\"" + code + "\" />\r\n")
			.append(XMLGenerator.MakeXMLTail());
			
			logger.info(sb.toString());
			
			SSLClient.sendToWallpad(targetIP, sb.toString());
			
			retMap.put("result", ERROR_INFO.CODE.SUCCESS.getContents());
			retMap.put("resultCode", ERROR_INFO.CODE.SUCCESS.getmIdx());
		}catch(Exception e) {
			retMap.put("result", ERROR_INFO.CODE.FAIL_999.getContents());
			retMap.put("resultCode", ERROR_INFO.CODE.FAIL_999.getmIdx());
			e.printStackTrace();
		}
		return retMap;
	}
	
	/**
	 * 기기 구독 / 해제 에 대한 메시지를 월패드에 전송한다. 
	 * 기기 구독이 돼 있지 않으면 이벤트를 받지 못한다.
	 * @param param
	 * @return
	 */
	public Map<String, Object> subscriptionProcess(Map<String, Object> param) {
		Map<String, Object> retMap = new HashMap<String, Object>();
		
		try {
			Document doc = new WallpadMsgSender().remoteDeviceSubscription(param);
			
			if(doc == null) {
				logger.error("[subscriptionProcess ERROR] - Document null");
				retMap.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_UNREACHABLE));
				return retMap;
			}
			
			retMap.put("result", "success");
		}catch(Exception e) {
			logger.error("[subscriptionProcess ERROR] - " + e.getStackTrace());
			retMap.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_INTERNAL));
		}
		return retMap;
	}
	
	/**
	 * 월패드와 연결된 기기들의 전체 상태를 월패드에 요청하고 응답 받는다.
	 * 지원 기기가 추가되면 해당 부분은 수정이 필요하다. 
	 * 자식 노드로 subdevinfo가 있는 기기들을 잘 확인해야 한다.
	 * @param dong
	 * @param ho
	 * @return
	 */
	public Map<String, Object> getDevicesStatusList(int dong, int ho) {
		Map<String, Object> retMap = new HashMap<String, Object>();
		Map<String, Object> body = new HashMap<String, Object>();
		List<String> unreachableIds = new ArrayList<String>();
		
		try {
			Document doc = new WallpadMsgSender().remoteDeviceStatusList(dong, ho);
			
			if(doc == null) {
				retMap.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_UNREACHABLE));
				logger.error("[ERROR]-[Document is Null] - [" + dong + "-" + ho + "]");
				return retMap;
			}
			
			XPath xpath = XPathFactory.newInstance().newXPath();
			Node iMap = (Node)xpath.evaluate("//imap/service", doc, XPathConstants.NODE);
			String xmlResult = iMap.getAttributes().getNamedItem("result").getTextContent();
			
			logger.debug("[KakaoMsgHandler - getDeviceList] - xmlResult -> " + xmlResult);
			
			if(!xmlResult.equalsIgnoreCase("ok")) {
				retMap.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_UNREACHABLE));
				logger.error("[KakaoMsgHandler - getDeviceList] xmlResult not ok");
				return retMap;
			}
			
			// 월패드와 연결된 상위 THINGS 갯수 확인
			Node dCount =  null;
	        
			NodeList devList = (NodeList)xpath.evaluate("//imap/service/devinfo", doc, XPathConstants.NODESET);
			for(int i=0; i < devList.getLength(); i++) {
				Node pNode = devList.item(i);
				
				String tmpModelId = pNode.getAttributes().getNamedItem("name").getTextContent();
				int childCount = Integer.parseInt(pNode.getAttributes().getNamedItem("value").getTextContent());
				
				// 아직 카카오에서 electric과 batchlight는 지원하지 않는다.
				// 나중에 지원하게 되면 그때 추가하면 된다.
				if(tmpModelId.equalsIgnoreCase("electric") || tmpModelId.equalsIgnoreCase("batchlight")) continue;
				
				List<Map<String, Object>> childNodeInfoList = null;
				
				// subdevinfo 를 가지고 있는 THINGS일 경우는 subdevinfo 들을 다시 구해야 한다.
				// 예를들어 light는 light01 ~ 05 등의 subdevinfo를 가진다.
				if(tmpModelId.equalsIgnoreCase("light")) {
					for(int j=1; j <= childCount; j++) {
						String childName = j < 10 ? tmpModelId+"0"+j : tmpModelId+j;
						Node childNode = (Node)xpath.evaluate("//imap/service/devinfo/*[@name='"+ childName +"']", doc, XPathConstants.NODE);
						
						childCount = Integer.parseInt(childNode.getAttributes().getNamedItem("value").getTextContent());
						
						NodeList childList = (NodeList)xpath.evaluate("//imap/service/devinfo/subdevinfo/*[@name='"+ childName +"']", doc, XPathConstants.NODESET);
						childNodeInfoList = getChildDeviceInfo(childCount, childList);
						
						if(childNodeInfoList != null)
							for(Map<String, Object> tMap : childNodeInfoList) body.putAll(tMap);
					}
					continue;
				}
				// 해당 THINGS의 자식노드 status 목록들을 조회한다.
				NodeList childList = (NodeList)xpath.evaluate("//imap/service/devinfo/*[@name='"+ tmpModelId +"']", doc, XPathConstants.NODESET);
				
				
				childNodeInfoList = getChildDeviceInfo(childCount, childList);
				
				if(childNodeInfoList != null)
					for(Map<String, Object> tMap : childNodeInfoList) body.putAll(tMap);
	        }
		}catch(Exception e) {
			retMap.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_INTERNAL));
			logger.error("[ERROR] - " + e.getMessage());
		}finally {
			retMap.put(KEY_INFO.HTTP_KEY_UNREACHABLEIDS, unreachableIds);
			retMap.put(KEY_INFO.HTTP_KEY_BODY, body);
		}
		
		return retMap;
	}
	/**
	 * 동 호 정보 EndpointID 로 월패드에 상태 조회 메세지를 전송하고 결과를 반환 받는다. 
	 * @param dong
	 * @param ho
	 * @param endpointId
	 * @return
	 */
	public Map<String, Object> getDeviceStatus(int dong, int ho, String endpointId){
		Map<String, Object> retMap = new HashMap<String, Object>();
		String modelId = "";
		int devNum = 1;

		Document doc = null;

		// endpoint Id로 존재하는 THINGS 인지 확인 
		// 발생 ERROR FAIL_NONEXIST
		try {
			String[] tmpSplit = endpointId.split("_");
			
			modelId = tmpSplit[0];
			devNum = Integer.parseInt(tmpSplit[1]);	// 회로 번호
			
			if(!THINGS_INFO.isExistThings(modelId)) {
				retMap.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_NONEXIST));
				logger.error("[ERROR]-[(" + endpointId + ") - Non Exist]");
				return retMap;
			}
		}catch(Exception e) { 
			retMap.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_NONEXIST));
			logger.error("[ERROR]-[(" + endpointId + ") - Non Exist]");
			return retMap;
		}
		
		try {
			doc = new WallpadMsgSender().remoteDeviceStatus(dong, ho, modelId, devNum);
			
			if(doc == null) {
				retMap.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_UNREACHABLE));
				logger.error("[ERROR]-[Document is Null] - [" + dong + "-" + ho + "] ["+ modelId +"]");
				return retMap;
			}
		}catch(Exception e) { 
			retMap.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_NONEXIST));
			logger.error("[ERROR]-[(" + endpointId + ") - Non Exist]");
			return retMap;
		}
		
		return getStatusMap(endpointId, doc);
	}
	
	public Map<String, Object> getDeviceStatus_bak(int dong, int ho, String endpointId){
		Map<String, Object> retMap = new HashMap<String, Object>();
		Document doc = null;

		try {
			String[] tmpSplit = endpointId.split("_");
			
			String modelId = tmpSplit[0];				// 기기 ID
			int devNum = Integer.parseInt(tmpSplit[1]);	// 회로 번호
			
			doc = new WallpadMsgSender().remoteDeviceStatus(dong, ho, modelId, devNum);
			
			if(doc == null) {
				retMap.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_UNREACHABLE));
				logger.error("[ERROR]-[Document is Null] - [" + dong + "-" + ho + "] ["+ modelId +"]");
				return retMap;
			}
		}catch(Exception e) { 
			retMap.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_NONEXIST));
			logger.error("[ERROR]-[(" + endpointId + ") - Non Exist]");
			return retMap;
		}
		
		return getStatusMap(endpointId, doc);
	}
	
	/**
	 * 월패드에 장치 제어 명령을 보내고 결과를 리턴한다.
	 * @param dong
	 * @param ho
	 * @param endpointId
	 * @param actionName
	 * @param desired
	 * @return
	 */
	public Map<String, Object> sendDeviceAction(int dong, int ho, String endpointId, String actionName, Map<String, Object> desired){
		Map<String, Object> retMap = new HashMap<String, Object>();
		
		String[] tmpSplit = endpointId.split("_");
		String modelId = tmpSplit[0];
		
		if(tmpSplit.length == 1 && !modelId.equals(THINGS_INFO.THINGS.ELEVATOR.getKakaoDisplayName())) {
			retMap.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_NONEXIST));
			return retMap;
		}
		
		int devNum = modelId.equals(THINGS_INFO.THINGS.ELEVATOR.getKakaoDisplayName())?1:Integer.parseInt(tmpSplit[1]);
		
		THINGS_INFO.THINGS things = THINGS_INFO.getThingsType(endpointId);
		
		if(things == null) {
			retMap.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_INVALID_REQ));
			return retMap;
		}
		String action = KakaoMsgHandler.setDeviceActionStr(modelId, actionName, desired);
		
		if(action.length() == 0) {
			retMap.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_INVALID_REQ));
			return retMap;
		}
		
		Document doc = new WallpadMsgSender().remoteDeviceControl(
																dong, 
																ho, 
																modelId, 
																devNum, 
																action
															);
		
		if(doc == null) {
			retMap.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_UNREACHABLE));
			return retMap;
		}
		
		return getStatusMap(endpointId, doc);
	}
	
	/**
	 * 상태조회 요청 메세지에 대한 응답 메세지를 파싱하여 카카오 형식으로 변환한다.
	 * @param endpointId
	 * @param doc
	 * @return
	 */
	public static Map<String, Object> getStatusMap(String endpointId, Document doc){
		Map<String, Object> retMap = new HashMap<String, Object>();
		
		XPath xpath = XPathFactory.newInstance().newXPath();
		try {
			Node iMap = (Node) xpath.evaluate("//imap/service", doc, XPathConstants.NODE);
			String xmlResult = iMap.getAttributes().getNamedItem("result").getTextContent();
			
			if(!xmlResult.equalsIgnoreCase("ok")) {
				retMap.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_INVALID_REQ)); 
				logger.error("[ERROR]-[xmlResult is not ok");
				return retMap;
			}
			
			String status = (String) xpath.evaluate("//imap/service/status", doc, XPathConstants.STRING);
			
			Map<String, Object> state = setDeviceState(endpointId, status);
			retMap.put(endpointId, state);
			
		} catch (XPathExpressionException e) {
			retMap.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_INTERNAL));
			e.printStackTrace();
		}
		return retMap;
	}
	
	
	/**
	 * 이벤트 발생시 카카오에 전송하는 Body를 반환한다.
	 * @param dong
	 * @param ho
	 * @param body
	 * @return
	 * @throws Exception
	 */
	public Map<String, Object> setKakaoEventMsg(DonghoInfo donghoInfo, Map<String, Object> body) throws Exception{
		Map<String, Object> retMap = new HashMap<String, Object>();
		List<String> unreachableIds = new ArrayList<>();
		
		retMap.put(KEY_INFO.HTTP_KEY_ID, UUID.randomUUID().toString()); 
		retMap.put(KEY_INFO.HTTP_KEY_TYPE, TYPE.REQ_TYPE.REPORTSTATE.getTypeName());
		retMap.put(KEY_INFO.HTTP_KEY_AUTHORIZATION, donghoInfo.getKakaoAccessToken());
		retMap.put(KEY_INFO.HTTP_KEY_VERSION, "1");
		retMap.put(KEY_INFO.HTTP_KEY_UNREACHABLEIDS, unreachableIds);
		retMap.put(KEY_INFO.HTTP_KEY_BODY, body);
		
		return retMap;
	}
	
	/**
	 * 지원하는 기기들의 Capability 를 카카오 형식에 맞게 추출한다.
	 * 기기가 추가되게 되면 목록을 수정해야한다.
	 * @param modelId
	 * @return
	 */
	public static Map<String, Object> getDeviceCapabilities(String modelId){
		THINGS_INFO.THINGS things = THINGS_INFO.getThingsType(modelId);
		
		String jsonString = "";
		
		switch(things.getEndPoint()) {
		case THINGS_INFO.THINGS_LIGHT:
			jsonString = THINGS_INFO.THINGS_CAPABILITY.LIGHT.getCapJsonStr();
			break;
			
		case THINGS_INFO.THINGS_TEMPER:
			jsonString = THINGS_INFO.THINGS_CAPABILITY.TEMPER.getCapJsonStr();
			break;
			
		case THINGS_INFO.THINGS_GAS:
			jsonString = THINGS_INFO.THINGS_CAPABILITY.GAS.getCapJsonStr();
			break;
			
		case THINGS_INFO.THINGS_VENTIL:
			jsonString = THINGS_INFO.THINGS_CAPABILITY.VENTIL.getCapJsonStr();
			break;
		case THINGS_INFO.THINGS_ELECPLUG:
			break;
		}
		
		Map<String, Object> capabilities = CommonUtil.jsonToMap(jsonString);
		
		return capabilities;
	}
	
	/**
	 * Document에서 추출한 Device의 자식 노드들을 추출한다.
	 * @param childCount
	 * @param childList
	 * @return
	 */
	public List<Map<String, Object>> getChildDeviceInfo(int childCount, NodeList childList){
		List<Map<String, Object>> deviceList = new ArrayList<>();
		
		for(int j=0; j < childCount; j++) {
			Map<String, Object> deviceMap = new HashMap<>();
			
			Node cNode = childList.item(j);
			if(cNode == null) {
				logger.error("cNode is null");
				return null;
			}
			
			String cNodeName = cNode.getAttributes().getNamedItem("name").getTextContent();
			String cNodeDevNum = cNode.getAttributes().getNamedItem("dev_num").getTextContent();
			String status = cNode.getTextContent();
			String endpointId = cNodeName+"_"+cNodeDevNum;
			
			THINGS_INFO.THINGS things = THINGS_INFO.getThingsType(cNodeName);
			
			Map<String, Object> deviceInfo =  new HashMap<String, Object>();
			
			deviceInfo.put(KEY_INFO.BODY_KEY_ENDPOINT_ID, endpointId);
    		deviceInfo.put(KEY_INFO.BODY_KEY_NAME, cNodeName.toUpperCase() + " " + cNodeDevNum);
    		deviceInfo.put(KEY_INFO.BODY_KEY_DISPLAY_TYPE, things.getKakaoDisplayName());
    		
    		deviceInfo.putAll(getDeviceCapabilities(endpointId));
    		deviceInfo.put(KEY_INFO.BODY_KEY_STATE, setDeviceState(endpointId, status));
    		
    		deviceMap.put(endpointId, deviceInfo);
    		deviceList.add(deviceMap);
		}
		
		return deviceList;
	}
	
	/**
	 * 디바이스의 상태 명령어를 디바이스 종류에 맞게 생성한다.
	 * @param modelId
	 * @param desired
	 * @return
	 */
	
	public static String setDeviceActionStr(String modelId, String actionName, Map<String, Object> desired) {
		StringBuffer sb = new StringBuffer();
		
		THINGS_INFO.THINGS things = THINGS_INFO.getThingsType(modelId);
		try {
			switch(things.getEndPoint()) {
			case THINGS_INFO.THINGS_LIGHT:
				if(actionName.equals("Power") && desired.containsKey("power")) {
					String tmpState = (String)desired.get("power");
					if(tmpState.equals("on") || tmpState.equals("off")) sb.append(tmpState);
				}
				break;
				
			case THINGS_INFO.THINGS_TEMPER:
				if(actionName.equals("Power") && desired.containsKey("power")) {
					String tmpState = (String)desired.get("power");
					if(tmpState.equals("on") || tmpState.equals("off")) sb.append(tmpState);
				}
					
				if(actionName.equals("TemperatureSetpoint") && desired.containsKey("targetSetpoint")) {
					double tempPoint = (double)desired.get("targetSetpoint");
					sb.append("on/").append(Double.toString(tempPoint));
				}
				
				break;
				
			case THINGS_INFO.THINGS_GAS:
				if(actionName.equals("Lock") && desired.containsKey("locked")){
					if((boolean)desired.get("locked"))
						sb.append("close");
				}
				break;
				
			case THINGS_INFO.THINGS_VENTIL:
				if(actionName.equals("Power") && desired.containsKey("power")) {
					String tmpState = (String)desired.get("power");
					if(tmpState.equals("on") || tmpState.equals("off")) sb.append(tmpState);
				}
				
				if(actionName.equals("AirFlow") && desired.containsKey("fanSpeed")) {
					String tmpFanSpeed = (String)desired.get("fanSpeed");
					
					if(tmpFanSpeed.equals("low") || tmpFanSpeed.equals("middle") || tmpFanSpeed.equals("high"))
						sb.append("on/").append(desired.get("fanSpeed"));
				}
				break;
				
			case THINGS_INFO.THINGS_ELECPLUG:
				if(desired.containsKey("power")) sb.append(desired.get("power"));
				else if(desired.containsKey("mode")) sb.append(desired.get("mode"));
				break;
				
			case THINGS_INFO.THINGS_ELEVATOR:
				if(actionName.equals("ElevatorCall") && desired.containsKey("callDirection")) {
					if("down".equalsIgnoreCase((String)desired.get("callDirection"))) {
						sb.append(desired.get("callDirection"));
					}
				}
			}
		}catch(Exception e) {
			
		}
		
		
		logger.info("[---->>]" + sb.toString());
		return sb.toString();
	}
	
	public static String setDeviceActionStr_bak(String modelId, Map<String, Object> desired) {
		StringBuffer sb = new StringBuffer();
		
		THINGS_INFO.THINGS things = THINGS_INFO.getThingsType(modelId);
		
		switch(things.getEndPoint()) {
		case THINGS_INFO.THINGS_LIGHT:
			sb.append(desired.get("power"));
			break;
			
		case THINGS_INFO.THINGS_TEMPER:
			if(desired.containsKey("power")) sb.append(desired.get("power"));
			else if(desired.containsKey("targetSetpoint")) sb.append("on/").append(desired.get("targetSetpoint"));
			
			break;
			
		case THINGS_INFO.THINGS_GAS:
			if(desired.containsKey("locked")){
				if((boolean)desired.get("locked"))
					sb.append("close");
			}
			break;
			
		case THINGS_INFO.THINGS_VENTIL:
			if(desired.containsKey("power")) sb.append(desired.get("power"));
			else if(desired.containsKey("fanSpeed")) sb.append("on/").append(desired.get("fanSpeed"));
			break;
			
		case THINGS_INFO.THINGS_ELECPLUG:
			if(desired.containsKey("power")) sb.append(desired.get("power"));
			else if(desired.containsKey("mode")) sb.append(desired.get("mode"));
			break;
			
		case THINGS_INFO.THINGS_ELEVATOR:
			if(desired.containsKey("callDirection")) {
				if("down".equalsIgnoreCase((String)desired.get("callDirection"))) {
					sb.append(desired.get("callDirection"));
				}
			}
		}
		
		logger.info("[---->>]" + sb.toString());
		return sb.toString();
	}
	
	/**
	 * 상태조회 이후 월패드에서 받은 상태 값을 디바이스의 속성에 맞게 파싱하고 
	 * Map에 <"속성명", "상태값"> 형태로 담아준다. 
	 * @param modelId
	 * @param status
	 * @return
	 */
	public static Map<String, Object> setDeviceState(String modelId, String status){
		Map<String,Object> state = new HashMap<String, Object>();
		
		THINGS_INFO.THINGS things = THINGS_INFO.getThingsType(modelId);
		String[] splitStr = status.split("\\/");
		
		switch(things.getEndPoint()) {
		case THINGS_INFO.THINGS_LIGHT:
			state.put("power", status);
			break;
		case THINGS_INFO.THINGS_TEMPER:
			
			state.put("power", splitStr[0]);
			state.put("targetSetpoint", Double.parseDouble(splitStr[1]));
			state.put("currentTemperature", Double.parseDouble(splitStr[2]));
			
			break;
		case THINGS_INFO.THINGS_GAS:
			if(status.equalsIgnoreCase("close"))
				state.put("locked", true);
			else
				state.put("locked", false);
			break;
			
		case THINGS_INFO.THINGS_VENTIL:
			state.put("power", splitStr[0]);
			if(splitStr[0].equalsIgnoreCase("on")) {
				if(splitStr[1].equals("mid")) state.put("fanSpeed", "middle");
				else state.put("fanSpeed", splitStr[1]);
			}
			break;
		case THINGS_INFO.THINGS_ELEVATOR:
			state.put("callDirection", "down");
			break;
		}
		
		return state;
	}
}
