package com.icontrols.kakao.socket.wallpad;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.Socket;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CompletableFuture;

import javax.annotation.Resource;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathFactory;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Node;

import com.icontrols.kakao.common.KAKAO_RESULT;
import com.icontrols.kakao.common.utils.BeanUtils;
import com.icontrols.kakao.common.utils.XmlUtil;
import com.icontrols.kakao.config.AsyncConfig;
import com.icontrols.kakao.entity.DonghoInfo;
import com.icontrols.kakao.repo.DonghoInfoRepository;

/**
 * 
 * @author 황의현 / 2019-03-13
 * 클라이언트로 데이터를 전송할 스레드 클래스
 * 
 * 월패드에서 전송한 이벤트 메세지를 받아서 처리를 하는 스레드 클래스
 *  
 */
public class ThreadServerHandler extends Thread {
	private static final Logger logger = LoggerFactory.getLogger(ThreadServerHandler.class);

	private Socket connectedClientSocket;					// 클라이언트와 연결된 소켓 객체
	
	private String strRcvMSG;								//  입력 받은 XML String 
	private Document doc;									// 
	
	static final int EVENTTIMELIMITE = 2000;
	static long lSocketConnectionPeriod = 0;
	static int TIMEOUT_iMAP_WAIT = 3000; // iMAP 타임아웃
	static int SOCKET_BUFFER_SIZE_READ = 50;
	static boolean newDanjiServerNoticeUpdate = false;

	
	// 생성자
	public ThreadServerHandler(Socket connectedClientSocket, boolean server) {
		
		// Client와 통신할 객체를 초기값으로 받아 할당합니다.
		this.connectedClientSocket = connectedClientSocket;
		this.newDanjiServerNoticeUpdate = server;
	}

	public void run() {
		try {
			// 클라이언트로 내용을 출력 할 객체 생성
			InputStream InStm = connectedClientSocket.getInputStream();									// Socket으로 연결된 클라이언트에게 Input Stream 을 얻는다.
			connectedClientSocket.setSoLinger(true, 3000);												// 소켓 옵션 설정 소켓이 닫힐 때 아직 전송되지 않은 데이터 처리에 대한 옵션 --> 3000초 의 linger time을 설정
			BufferedReader MsgReader = new BufferedReader(new InputStreamReader(InStm, "euc-kr"));
			strRcvMSG = "";
			
			StringBuffer sbRcvMSG = new StringBuffer(); 												// Client에게 수신받은 iMap 메시지를 append한 뒤에 strRcvMSG에 할당한다.
			StringBuffer sbInBuffer = null;
			
//			String InBuffer;
			
			boolean bRcvOK = false;
			lSocketConnectionPeriod = System.currentTimeMillis() + TIMEOUT_iMAP_WAIT; // 타임아웃시간을 설정한다.

			while (System.currentTimeMillis() < lSocketConnectionPeriod) {
				try {
					if (MsgReader.ready()) {
						try {
							char[] ReadData = new char[SOCKET_BUFFER_SIZE_READ * 1024];
							sbInBuffer = new StringBuffer();
							while (MsgReader.ready()) {
								long charCnt = MsgReader.read(ReadData);
								sbInBuffer.append(new String(ReadData, 0, (int) charCnt));
							}

							if (sbInBuffer != null || sbInBuffer.toString().trim().length() > 0)
								sbRcvMSG.append(sbInBuffer.toString());
							
							if (0 < sbInBuffer.toString().indexOf("</imap>")) { // </imap> 으로 제대로 닫혔다면 클라이언트의 이벤트 메세지를 제대로 수신 했다는 말.
								bRcvOK = true;
								logger.info("[iMAPSvrProc().run()] Received DONE!!");
								break;
							}
						} catch (Exception e) {
							logger.error("message : " + e);
							return;
						}
					} else {
						Thread.sleep(100);
					}
				} catch (Exception e) {
					logger.info("[Exception] iMAPSvrProc.run() ===== 2 =====");
					connectedClientSocket.close();
					logger.error("message : " + e);
					return;
				}finally {
					strRcvMSG = sbRcvMSG.toString();
				}
			}

			if (!bRcvOK) {
				logger.error("[iMAPSvrProc().run()] iMAP message is unacceptable!! :\n\r" + strRcvMSG);
				connectedClientSocket.close();
				return;
			}
			
			// 응답을 제대로 받은 이후의 처리 .. 제대로 된 응다블 받지 못했다면 위에서 소켓 연결 해제 
			logger.info("-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-");
			logger.info("strRcvMSG :: > " + strRcvMSG);
			
			strRcvMSG = strRcvMSG.substring(strRcvMSG.indexOf("<")); // 이 부분에 대한 예시 데이터가 있으면 이해하기 좋을거 같다. 테스트 하면서 뽑아내고 주석으로 잘아보자.
			
			
			// XML문서를 읽어보자구요!																		
			// XML을 Document라는 객체로 인식
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();						// XML 문서를 읽기 위한 Factory 객체 생성
			DocumentBuilder builder = factory.newDocumentBuilder();										// Builder 객체 생성
			InputStream istream = new ByteArrayInputStream(strRcvMSG.getBytes("UTF-8"));
			doc = builder.parse(istream);

			// xpath 생성
			XPath xpath = XPathFactory.newInstance().newXPath();

			// id 가 c2 인 Node의 val attribute 값 가져오기
			Node service = (Node) xpath.evaluate("//imap/service", doc, XPathConstants.NODE);
			String requestType = XmlUtil.getAttributeWithItem(service, "type");
			logger.info("[ThreadServerHandler]-service type : " + requestType);
			
			String retXML = "";
			if (requestType.equalsIgnoreCase("notice")) {
				String requestCmd = XmlUtil.getAttributeWithItem(service, "name");
				
				logger.info("[ThreadServerHandler]-requestCmd : " + requestCmd);
//				if (requestCmd.equalsIgnoreCase("msg_device_event")) { // 상태변경이벤트인경우에 처리
					logger.info("[ThreadServerHandler]-strRcvMSG : " + strRcvMSG);
					// 빈번하게 실행 될 가능성이 높다.
					DeviceEventMsgHandler emh = new DeviceEventMsgHandler(doc, requestCmd);
					emh.start();
//				}
			}
		} catch (Exception e) {
			try {
				connectedClientSocket.close();
			} catch (Exception E) {
				logger.error("message : " + E);
			}
			logger.error("message : " + e);
		}
	}

	// 클라이언트로 데이터를 전송할 스레드 클래스
	class DeviceEventMsgHandler extends Thread {
		// 멤버변수
		private Document doc;
		private String cmd;
		
		
		// 생성자
		public DeviceEventMsgHandler(Document doc, String cmd) {
			this.doc = doc;
		}

		public void run() {
			try {
				DonghoInfoRepository repo = (DonghoInfoRepository)BeanUtils.getBean("donghoInfoRepository");
				KakaoMsgHandler kakaoMsgHandler = (KakaoMsgHandler)BeanUtils.getBean("kakaoMsgHandler");
				KakaoEventHandler kakaoEventHandler = null;
				// 데이터 뽑아서 이통사 연동 서버에 전달한다.
				String model_id, status;
				int dev_num;

				// xpath 생성
				XPath xpath = XPathFactory.newInstance().newXPath();

				// id 가 c2 인 Node의 val attribute 값 가져오기
				Node iMap = (Node) xpath.evaluate("//imap", doc, XPathConstants.NODE);
				logger.info("[DeviceEventMsgHandler] - address : " + iMap.getAttributes().getNamedItem("address").getTextContent());
				logger.info("[DeviceEventMsgHandler] - sender : "  + iMap.getAttributes().getNamedItem("sender").getTextContent());


				String dongho = iMap.getAttributes().getNamedItem("sender").getTextContent();
				String searchDong = (dongho.indexOf("d") >= 0)?"d":"동";
				String searchHo = (dongho.indexOf("d") >= 0)?"h":"호";

				int idxdong = dongho.indexOf(searchDong);
				int idxho = dongho.indexOf(searchHo);

				String dong = dongho.substring(0, idxdong).trim();
				String ho = dongho.substring(idxdong + 1, idxho).trim();
				
				logger.info("[DeviceEventMsgHandler] - [동:" + dong + "호:" + ho + "]");

				Node service = (Node) xpath.evaluate("//imap/service", doc, XPathConstants.NODE);
				logger.info(service.getAttributes().getNamedItem("type").getTextContent());

				model_id = (String) xpath.evaluate("//imap/service/model_id ", doc, XPathConstants.STRING);
				dev_num = Integer.parseInt((String) xpath.evaluate("//imap/service/dev_num", doc, XPathConstants.STRING));
				status = (String) xpath.evaluate("//imap/service/status", doc, XPathConstants.STRING);
				
				logger.info("[DeviceEventMsgHandler] - model_id : " + model_id);
				logger.info("[DeviceEventMsgHandler] - dev_num  : " + dev_num);
				logger.info("[DeviceEventMsgHandler] - status   : " + status);

				String endpointId = model_id + "_" + dev_num;
				int int_dong = Integer.parseInt(dong);
				int int_ho = Integer.parseInt(ho);
				
				Map <String, Object> body = new HashMap<String, Object>();
				
				
				AsyncConfig asyncConfig = (AsyncConfig)BeanUtils.getBean("asyncConfig");
				
				if(!status.equalsIgnoreCase("unsubscription")) {
					//dong ho 가  같은 유저 모두 조회
					
					List<DonghoInfo> dList = repo.findByDongAndHo(dong, ho);
					kakaoEventHandler = (KakaoEventHandler)BeanUtils.getBean("kakaoEventHandler");
					
					List<CompletableFuture<Map<String, Object>>> fList = new ArrayList<>();
					
					for(DonghoInfo donghoInfo:dList) {
						// 삭제된 토큰은 이벤트 전송을 하지 않는다.
						if(donghoInfo.getDeleteFlag().equals("Y")) continue;
						body.put(endpointId, KakaoMsgHandler.setDeviceState(endpointId, status));
						logger.debug("body --> " + body.toString());
						
						Map<String, Object> sendMap = kakaoMsgHandler.setKakaoEventMsg(donghoInfo, body);
						logger.debug("sendMap --> " + sendMap.toString());
						
						while(true) {
			    			if(asyncConfig.isEventTaskExecute()) {
			    				fList.add(kakaoEventHandler.sendEventAsync(donghoInfo, sendMap));
			    				break;
			    			}
			    			Thread.sleep(1000);
			    		}
					}
					
					CompletableFuture.allOf(fList.toArray(new CompletableFuture[fList.size()])).join();
					
					// 이벤트 전송이 실패했는지 성공했는지..
					for(CompletableFuture<Map<String, Object>> future:fList) {
						Map<String, Object> eventResult = future.get();
						
						int code = (int)eventResult.get("result");
						String message = (String)eventResult.get("message");
						
						if(code == KAKAO_RESULT.SUCCESS) {
							logger.info("[EVENT SUCCESS] - " + message);
						} else {
							logger.error("[EVENT ERROR] - " + message);
						}
					}
				}
			} catch (Exception e) {
				logger.error("message : " + e);
				e.printStackTrace();
				
			}
		}

		private String trimquotation(String sen) {
			sen = sen.replaceAll("\"", "");
			return sen.trim();
		}
	}
}
